--The Only way to utilize multiple profiles is to save everything under a single db.
--We are currently only able to utilize multiple profiles on a server datascope basis without overwriting data (name conflicts).
--LibDB version 0.3

local setmetable = setmetatable;

DB = {}

local function New(self, datascope, key, defaults, defaultprofile ) 
        --create new database with key, datascope, defaults, and defaultprofile built in 
        --Warning!! If datascope is set to player, unable to share profile information.
	--If you use an account datascope, players that have toons with the same name will have their db overwritten unless ask the player for server information. 
	object = {}
	setmetatable( object, self );
      	self.__index = self;
        self.datascope = datascope; 
        self.key = key;	
	self.profiles = Turbine.PluginData.Load(datascope, key);
	if (self.profiles == nil ) then
		self.profiles = {};
	end	
	self:RegisterDefaults( defaultprofile )
	return object
end

local function deepcopy( object )
 	--taken from  http://lua-users.org/wiki/CopyTable
	--Copies entire profile to another profile.
	local lookup_table = {}
	local function _copy(object)
        if type(object) ~= "table" then
        	return object
        elseif lookup_table[object] then
		return lookup_table[object]
        end
        local new_table = {}
        lookup_table[object] = new_table
        for index, value in pairs(object) do
		new_table[_copy(index)] = _copy(value)
        end
        return setmetatable(new_table, getmetatable(object))
    end
    return _copy(object)
end

local function CopyProfile ( self, profile )
	return deepcopy( profile )
end

local function DeleteProfile(self, profile ) 
	--Ex: mydb:DeleteProfile( ProfileNameToBeDeleted );
        self.profiles[profile] = nil;
	self:SaveDB();
end

local function SetCurrentProfile(self, profile )
	--Sets the current active profile.
	--Can use for example when character wants to switch active profile
	self.profiles["ACTIVE"] = self.profiles[profile]["name"];
end

local function GetCurrentProfile( self )
	--Ex: mydb.profiles[ mydb.GetCurrentProfile() ]
	return self.profiles["ACTIVE"];
end

local function RegisterDefaults( self, defaults )
	--Registers the default profile that is used typically on first load of a plugin and saves the db.
	--This is called on database creation so should be setup there.
	--Ex: mydb:RegisterDefaults( MYDEFAULTSTABLE );
	if not self.profiles["ACTIVE"] then
		self.profiles["Defaults"] = defaults;
		self.profiles["Defaults"].name = "Defaults";
		self.profiles["ACTIVE"] = defaults["name"];
	end	
	self:SaveDB()
end

local function ResetDB( self, defaultprofile ) 
        --Resets entire db with just defaults left and saves the DB.
	local defaults = defaultprofile;
	self.profiles = nil;
	self.profiles = {};
	self.profiles["Defaults"] = defaults;
	self:RegisterDefaults( defaultprofile );	
	self:SaveDB();
end

local function ResetProfile( self ) 
        --reset current profile
	self.profiles["Active"] = self.profiles["Defaults"]["name"];
	self.SaveDB();
end

local function Destroy( self ) 
        --destroy object
        self = nil; 
end

local function SaveDB( self ) 
        Turbine.PluginData.Save( self.datascope, self.key, self.profiles ) 
end

local function NewProfile( self, name ) 
	self.profiles[name] = {};
	self.profiles[name] = deepcopy( self.profiles["Defaults"] );
	self.profiles[name]["name"] = name;
	return self.profiles.name        
end

DB = {
	New = New,
	CopyProfile = CopyProfile,
	DeleteProfile = DeleteProfile,
	SetCurrentProfile = SetCurrentProfile,
	GetCurrentProfile = GetCurrentProfile,	
	RegisterDefaults = RegisterDefaults,
	ResetDB = ResetDB,
	ResetProfile = ResetProfile,	
	Destroy = Destroy,
	SaveDB = SaveDB,
	NewProfile = NewProfile,	
}